from htmllib import HTMLParser
from formatter import AbstractFormatter, DumbWriter

class DocTranslator(HTMLParser):

    def __init__(self):
        HTMLParser.__init__(self, AbstractFormatter(DumbWriter()))
        self.in_par = 0

    def do_p(self, attrs):
        if self.in_par:
            self.formatter.add_literal_data('\n</para>\n')
        self.formatter.add_literal_data('<para>\n')
        self.in_par = 1

    def start_h1(self, attrs):
        if self.in_par:
            self.formatter.add_literal_data('\n</para>\n')
        self.formatter.add_literal_data('<chapter>\n')
        self.formatter.add_literal_data('<title>')
    def end_h1(self):
        self.formatter.add_literal_data('</title>\n')
        self.in_par = 0

    def start_h2(self, attrs):
        if self.in_par:
            self.formatter.add_literal_data('\n</para>\n')
        self.formatter.add_literal_data('<sect1>\n')
        self.formatter.add_literal_data('<title>')
    def end_h2(self):
        self.formatter.add_literal_data('</title>\n')
        self.in_par = 0

    def start_ul(self, attrs):
        self.formatter.add_literal_data('\n<itemizedlist>\n')
        self.in_item = 0
    def end_ul(self):
        if self.in_item:
            self.formatter.add_literal_data('\n</simpara></listitem>\n')
            self.in_item = 0
        self.formatter.add_literal_data('</itemizedlist>\n')
    def do_li(self, attrs):
        if self.in_item:
            self.formatter.add_literal_data('\n</simpara></listitem>\n')
        self.formatter.add_literal_data('<listitem><simpara>\n')
        self.in_item = 1

    def do_hr(self, attrs):
        pass

    def start_pre(self, attrs):
        self.formatter.add_literal_data('\n<programlisting role="Python">')
        self.nofill = 1
    def end_pre(self):
        self.formatter.add_literal_data('</programlisting>\n')
        self.nofill = 0
    def start_tt(self, attrs):
        self.formatter.add_literal_data('<literal role="Python">')
    def end_tt(self):
        self.formatter.add_literal_data('</literal>')

    def start_a(self, attrs):
        href = None
        self.in_a = 0
        for a in attrs:
            if a[0] == 'href': href = a[1]
        if href:
            self.formatter.add_literal_data('<ulink URL="%s">' % href)
            self.in_a = 1
    def end_a(self):
        if self.in_a:
            self.formatter.add_literal_data('</ulink>')
        
if __name__ == '__main__':

    import sys
    filename = sys.argv[1]
    t = DocTranslator()
    t.feed(open(filename).read())
    t.close()
