# This file provides the Amber force field, using Amber parameter files.
#
# Written by Konrad Hinsen
# last revision: 1999-2-17
#

from MMTK.ForceFields import MMForceField
import AmberData

#
# The total force field
#
class AmberForceField(MMForceField.MMForceField):

    def __init__(self, lj_options = None, es_options = None):
	self.arguments = (lj_options, es_options)
        MMForceField.MMForceField.__init__(self, 'Amber94', AmberData.Amber94,
                                           lj_options, es_options)

Amber94ForceField = AmberForceField

class Amber91ForceField(MMForceField.MMForceField):

    def __init__(self, lj_options = None, es_options = None):
	self.arguments = (lj_options, es_options)
        MMForceField.MMForceField.__init__(self, 'Amber91', AmberData.Amber91,
                                           lj_options, es_options)

#
# The following classes provides access to individual terms of the
# Amber force field. They are not used normally, but can be useful
# for testing.
#

#
# Bonded interactions
#
class AmberBondedForceField(MMForceField.MMBondedForceField):

    def __init__(self):
	MMForceField.MMBondedForceField.__init__(self, 'Amber_bonded',
                                                 AmberData.Amber94)

#
# Nonbonded interactions
#
class AmberLJForceField(MMForceField.MMLJForceField):

    def __init__(self, cutoff = None):
	MMForceField.MMLJForceField.__init__(self, 'Amber_LJ',
                                             AmberData.Amber94, cutoff)

class AmberESForceField(MMForceField.MMESForceField):

    def __init__(self, cutoff = None):
	MMForceField.MMESForceField.__init__(self, 'Amber_ES',
                                             AmberData.Amber94, cutoff)

class AmberEwaldESForceField(MMForceField.MMEwaldESForceField):

    def __init__(self, options = {}):
	MMForceField.MMEwaldForceESField.__init__(self, 'Amber_ES',
                                                  AmberData.Amber94, options)

class AmberMPESForceField(MMForceField.ESMPForceField):

    def __init__(self, options = {}):
	MMForceField.MMMPESForceField.__init__(self, 'Amber_ES',
                                               AmberData.Amber94, options)

class AmberLJESForceField(MMForceField.MMLJESForceField):

    def __init__(self):
	MMForceField.MMLJESForceField.__init__(self, 'Amber_LJES',
                                               AmberData.Amber94)

class AmberNonbondedForceField(MMForceField.MMNonbondedForceField):

    def __init__(self, lj_options = None, es_options = None):
        MMForceField.MMNonbondedForceField.__init__(self, AmberData.Amber94,
                                                    lj_options, es_options)
