# Deformation force field
#
# Written by Konrad Hinsen
# last revision: 1999-3-4
#

from ForceField import ForceField, ForceFieldData, ForceFieldEvaluator
from MMTK_forcefield import NonbondedList, NonbondedListEvaluator, \
                            CombinedEvaluator
from MMTK_deformation import DeformationEvaluator
import Numeric

#
# The deformation force field class
#
class DeformationForceField(ForceField):

    def __init__(self, fc_length = 0.7, cutoff = 1.2, factor = 46401.746122):
	ForceField.__init__(self, 'deformation')
	self.arguments = (fc_length, cutoff, factor)
	self.fc_length = fc_length
	self.cutoff = cutoff
	self.factor = factor

    def ready(self, global_data):
	return 1

    def evaluator(self, universe, subset1, subset2, global_data):
	if subset1 is not None:
	    for s1, s2 in [(subset1, subset2), (subset2, subset1)]:
		set = {}
		for a in s1.atomList():
		    set[a.index] = None
		for a in s2.atomList():
		    try:
			del set[a.index]
		    except KeyError: pass
	    set = {}
	    for a in subset1.atomList():
		set[a.index] = None
	    for a in subset2.atomList():
		set[a.index] = None
	    atom_subset = set.keys()
	    atom_subset.sort()
	    atom_subset = Numeric.array(atom_subset)
	else:
	    atom_subset = Numeric.array([], Numeric.Int)
	nothing = Numeric.zeros((0,2), Numeric.Int)
	nbl = NonbondedList(nothing, nothing, atom_subset, universe._spec,
                            self.cutoff)
	update = NonbondedListEvaluator(nbl)
	ev = DeformationEvaluator(universe._spec, nbl, self.fc_length,
				  self.cutoff, self.factor, 1.)
	return CombinedEvaluator(Numeric.array([update, ev]))
