/* Private include file for C force field calculations.
 *
 * Written by Konrad Hinsen
 * last revision: 1999-3-4
 */

#ifndef MMTK_FORCEFIELD_PRIVATE_H

#include "mmtk_universe.h"


/* Global variables */

extern distance_fn *distance_vector_pointer;
extern distance_fn *orthorhombic_distance_vector_pointer;

/* Functions in _forcefieldmodule.c */

void
add_pair_fc(PyFFEvaluatorObject *eval, PyObject *fc, fc_function *fc_fn,
	    int i, int j, vector3 dr, double r, double f1, double f2);

void
default_add_energy(PyObject *ev, PyObject *dict);

void
combined_add_energy(PyObject *ev, PyObject *dict);

/* Functions in bonded.c */

ff_eval_function harmonic_bond_evaluator;
ff_eval_function harmonic_angle_evaluator;
ff_eval_function cosine_dihedral_evaluator;


/* Functions in nonbonded.c */

int
nblist_update(PyNonbondedListObject * nblist, int natoms, double *coordinates);

ff_eval_function nonbonded_evaluator;
ff_eval_function lennard_jones_evaluator;
ff_eval_function electrostatic_evaluator;
ff_eval_function es_mp_evaluator;
ff_eval_function lj_es_evaluator;
ff_eval_function es_ewald_evaluator;


/* Functions in ewald.c */

static double
reciprocal_sum(int natoms, vector3 *x, double volume,
	       double *charge, double beta,
	       long *kmax, double cutoff_sq,
	       box_fn *box_transformation_fn, double *universe_data,
	       PyObject *gradients, gradient_function *gradient_fn,
	       PyObject *force_constants, fc_function *fc_fn,
	       PyFFEvaluatorObject *self,
	       void *scratch);

int
init_kvectors(box_fn *box_transformation_fn, double *universe_data, int natoms,
	      long *kmax, double cutoff_sq, void *scratch, int nvect);

/* Functions in sparsefc.c */

PyObject *
SparseForceConstants(PyObject *dummy, PyObject *args);

fc_function sparse_fc_function;

#define MMTK_FORCEFIELD_PRIVATE_H
#endif
