/* Include file for C universe-related functions.
 *
 * Written by Konrad Hinsen
 * last revision: 1999-3-23
 */

#ifndef MMTK_UNIVERSE_H

#include "mmtk.h"

/* Type definitions */

typedef void distance_fn(vector3 d, vector3 r1, vector3 r2, double *data);
typedef void correction_fn(vector3 *x, int natoms, double *data);
typedef double volume_fn(double scale_factor, double *data);
typedef void box_fn(vector3 *x, vector3 *b, int n, double *data, int to_box);
typedef void bounding_box_fn(vector3 *box1, vector3 *box2, vector3 *x,
			     int n, double *data);


/* Universe specification object structure */

typedef struct {
  PyObject_HEAD
  PyArrayObject *geometry;
  double *geometry_data;
  distance_fn *distance_function;
  correction_fn *correction_function;
  volume_fn *volume_function;
  box_fn *box_function;
  bounding_box_fn *bounding_box_function;
  int is_periodic;
} PyUniverseSpecObject;


/* Macro definitions */

#define distance_vector_1(d, r1, r2, data) \
  { \
    d[0] = r2[0]-r1[0]; \
    d[1] = r2[1]-r1[1]; \
    d[2] = r2[2]-r1[2]; \
  }

#define distance_vector_2(d, r1, r2, data) \
  { \
    d[0] = r2[0]-r1[0]; \
    if (d[0] > data[3]) d[0] -= data[0]; \
    if (d[0] < -data[3]) d[0] += data[0]; \
    d[1] = r2[1]-r1[1]; \
    if (d[1] > data[4]) d[1] -= data[1]; \
    if (d[1] < -data[4]) d[1] += data[1]; \
    d[2] = r2[2]-r1[2]; \
    if (d[2] > data[5]) d[2] -= data[2]; \
    if (d[2] < -data[5]) d[2] += data[2]; \
  }


/*
 * C API
 */

#define PyUniverseSpec_Type_NUM 0

/* Total number of C API pointers */
#define PyUniverse_API_pointers 1


#ifdef _UNIVERSE_MODULE

/* Type object declarations */
extern PyTypeObject PyUniverseSpec_Type;

/* Type check macros */
#define PyUniverseSpec_Check(op) ((op)->ob_type == &PyUniverseSpec_Type)

/* C API function declarations */

#else

/* C API address pointer */ 
static void **PyUniverse_API;

/* Type definitions */
#define PyUniverseSpec_Type *(PyTypeObject *) \
                             PyUniverse_API[PyUniverseSpec_Type_NUM]

/* Type check macros */
#define PyUniverseSpec_Check(op) \
   ((op)->ob_type == (PyTypeObject *)PyUniverse_API[PyUniverseSpec_Type_NUM])

/* C API function declarations */

#endif

#define MMTK_UNIVERSE_H
#endif
