#include"Python.h"
#include"arrayobject.h"

typedef struct { float r, i; } f2c_complex;
typedef struct { double r, i; } f2c_doublecomplex;
typedef long int (*L_fp)();

static PyObject *ErrorObject;

static PyObject *LapackError()
{  if (! ErrorObject)
      ErrorObject = PyString_FromString("LapackError");
   Py_INCREF(ErrorObject);
   return ErrorObject;
}

static PyObject *ErrorReturn(char *mes)
{  if (!ErrorObject)
      ErrorObject = PyString_FromString("LapackError");
   PyErr_SetString(ErrorObject,mes);
   return NULL;
}

#define TRY(E) if( ! (E)) return NULL

static int lapack_dsyev_CheckObject(PyObject *ob, int t, char *obname,
	char *tname, char *funname)
{	char buf[255];
	if (! PyArray_Check(ob))
	{	sprintf(buf,"Expected an array for parameter %s in lapack_dsyev.%s",obname, funname);
        ErrorReturn(buf);
        return 0;
    }
    if (!(((PyArrayObject *)ob)->flags & CONTIGUOUS)) 
    {    sprintf(buf,"Parameter %s is not contiguous in lapack_dsyev.%s",obname, funname);
        ErrorReturn(buf);
        return 0;
    }
    if (!(((PyArrayObject *)ob)->descr->type_num == t))
    {    sprintf(buf,"Parameter %s is not of type %s in lapack_dsyev.%s",obname,tname, funname);
        ErrorReturn(buf);
        return 0;
    }
    return 1;
}

#define LDATA(p) ((long int *) (((PyArrayObject *)p)->data))
#define CHDATA(p) ((char *) (((PyArrayObject *)p)->data))
#define SHDATA(p) ((short int *) (((PyArrayObject *)p)->data))
#define DDATA(p) ((double *) (((PyArrayObject *)p)->data))
#define FDATA(p) ((float *) (((PyArrayObject *)p)->data))
#define CDATA(p) ((f2c_complex *) (((PyArrayObject *)p)->data))
#define ZDATA(p) ((f2c_doublecomplex *) (((PyArrayObject *)p)->data))




static PyObject *lapack_dsyev_dsyev(PyObject *self, PyObject *args)
{
int  lapack_dsyev_status__;
char jobz;
char uplo;
long int n;
PyObject *a;
long int lda;
PyObject *w;
PyObject *work;
long int lwork;
long int info;
TRY(PyArg_ParseTuple(args,"cciOiOOii",&jobz,&uplo,&n,&a,&lda,&w,&work,&lwork,&info));

TRY(lapack_dsyev_CheckObject(a,PyArray_DOUBLE,"a","PyArray_DOUBLE","dsyev")); 
TRY(lapack_dsyev_CheckObject(w,PyArray_DOUBLE,"w","PyArray_DOUBLE","dsyev")); 
TRY(lapack_dsyev_CheckObject(work,PyArray_DOUBLE,"work","PyArray_DOUBLE","dsyev")); 


#if defined(NO_APPEND_FORTRAN)
lapack_dsyev_status__ = dsyev(&jobz,&uplo,&n,DDATA(a),&lda,DDATA(w),DDATA(work),&lwork,&info);
#else
lapack_dsyev_status__ = dsyev_(&jobz,&uplo,&n,DDATA(a),&lda,DDATA(w),DDATA(work),&lwork,&info);
#endif

return Py_BuildValue("{s:i,s:c,s:c,s:i,s:i,s:i,s:i}","dsyev_",lapack_dsyev_status__,"jobz",jobz,"uplo",uplo,"n",n,"lda",lda,"lwork",lwork,"info",info);
}


static struct PyMethodDef lapack_dsyev_module_methods[] = {
{"dsyev",lapack_dsyev_dsyev,1},
{ NULL,NULL,0}
};

static PyObject *lapack_dsyevError;
void initlapack_dsyev()
{ PyObject *m,*d;
m = Py_InitModule("lapack_dsyev",lapack_dsyev_module_methods);
#ifdef import_array
import_array();
#endif
d = PyModule_GetDict(m);
ErrorObject = LapackError();
PyDict_SetItemString(d,"LapackError",ErrorObject);
}
